"""Meta client for interacting with the Meta API.

Note: we use the OpenAI client to interact with the Meta models.

Only azure deployments are supported for now.
"""

from typing import Any

from openai import AsyncOpenAI, OpenAI

from llmax.external_clients.exceptions import ProviderNotFoundError
from llmax.models import Deployment

Client = Any


def get_client(deployment: Deployment) -> Client:
    """Get a Meta client for the given deployment."""
    match deployment.provider:
        case "azure":
            return OpenAI(
                api_key=deployment.api_key,
                base_url=deployment.endpoint,
            )
        case _:
            raise ProviderNotFoundError(deployment)


def get_aclient(deployment: Deployment) -> Client:
    """Get an async Meta client for the given deployment."""
    match deployment.provider:
        case "azure":
            return AsyncOpenAI(
                api_key=deployment.api_key,
                base_url=deployment.endpoint,
            )
        case _:
            raise ProviderNotFoundError(deployment)



"""Supported models."""

from typing import Literal, Union, get_args

CohereModel = Literal[
    "command-r",
    "command-r-plus",
]

MetaModel = Literal["llama-3-70b-instruct",]

GeminiModel = Literal["google/gemini-1.5-flash-002", "google/gemini-1.5-pro-002"]

MistralModel = Literal[
    "mistral-large",
    "mistral-small",
    "mistral-large-2411",
]


OpenAIModel = Literal[
    "ada-v2",
    "gpt-3.5",
    "gpt-4",
    "gpt-4-turbo",
    "gpt-4o",
    "text-embedding-3-large",
    "text-embedding-3-small",
    "whisper-1",
    "gpt-4o-mini",
    "o3-mini",
    "o1-preview",
    "dall-e-3",
    "tts-1",
]

AnthropicModel = Literal[
    "claude-3.5-sonnet",
    "claude-3-haiku",
]


LLModel = Union[
    CohereModel,
    MetaModel,
    MistralModel,
    OpenAIModel,
    AnthropicModel,
    GeminiModel,
]

SpeechToTextModel = Literal["whisper-1",]

ImageGenerationModel = Literal["dall-e-3",]

TextToSpeechModel = Literal["tts-1",]

Model = Union[LLModel, SpeechToTextModel, ImageGenerationModel]

COHERE_MODELS: tuple[CohereModel, ...] = get_args(CohereModel)
META_MODELS: tuple[MetaModel, ...] = get_args(MetaModel)
GEMINI_MODELS: tuple[GeminiModel, ...] = get_args(GeminiModel)
MISTRAL_MODELS: tuple[MistralModel, ...] = get_args(MistralModel)
OPENAI_MODELS: tuple[OpenAIModel, ...] = get_args(OpenAIModel)
ANTHROPIC_MODELS: tuple[AnthropicModel, ...] = get_args(AnthropicModel)

LLMS: tuple[LLModel, ...] = get_args(LLModel)
AUDIO: tuple[SpeechToTextModel, ...] = get_args(SpeechToTextModel)
IMAGE: tuple[ImageGenerationModel, ...] = get_args(ImageGenerationModel)
TEXTTOAUDIO: tuple[TextToSpeechModel, ...] = get_args(TextToSpeechModel)

MODELS: tuple[Model, ...] = get_args(Model)



"""Completion model for universal client."""

from __future__ import annotations

from typing import TYPE_CHECKING, Any, Callable, Generator, Iterable

if TYPE_CHECKING:
    from openai.types.chat import ChatCompletion, ChatCompletionChunk

    from llmax.external_clients.universal_client.chat_completion_message import (
        ChatCompletionMessage,
    )
    from llmax.models.deployment import Deployment
    from llmax.models.models import Model


class Completions:
    """Completions for the universal client."""

    client: Any
    completion_call: Callable[
        ...,
        ChatCompletion | None | Generator[ChatCompletionChunk, None, None],
    ]
    deployment: Deployment

    def __init__(
        self,
        client: Any,
        completion_call: Callable[
            ...,
            ChatCompletion | None | Generator[ChatCompletionChunk, None, None],
        ],
        deployment: Deployment,
    ) -> None:
        """Construct a completions object based on OpenAI client model."""
        self.client = client
        self.completion_call = completion_call
        self.deployment = deployment

    def create(
        self,
        messages: Iterable[ChatCompletionMessage],
        model: Model,  # noqa: ARG002
        *args: Any,
        **kwargs: Any,
    ) -> ChatCompletion | Generator[ChatCompletionChunk, None, None] | None:
        """Create function to create a completion item."""
        return self.completion_call(
            self.client,
            messages,
            self.deployment.deployment_name,
            *args,
            **kwargs,
        )



"""Gemini client for interacting with the Cohere API.

Note: we use the OpenAI client to interact with the Cohere models.
"""

from typing import Any

from google.auth.transport.requests import Request
from openai import AsyncOpenAI, OpenAI

from llmax.external_clients.exceptions import ProviderNotFoundError
from llmax.models import Deployment

Client = Any


def get_client(deployment: Deployment) -> Client:
    """Get a Gemini client for the given deployment."""
    match deployment.provider:
        case "openai":
            if deployment.creds:
                auth_req = Request()
                deployment.creds.refresh(auth_req)
                return OpenAI(
                    api_key=deployment.creds.token,
                    base_url=deployment.endpoint,
                )
            raise ProviderNotFoundError(deployment)
        case _:
            raise ProviderNotFoundError(deployment)


def get_aclient(deployment: Deployment) -> Client:
    """Get an async Gemini client for the given deployment."""
    match deployment.provider:
        case "openai":
            if deployment.creds:
                auth_req = Request()
                deployment.creds.refresh(auth_req)
                return AsyncOpenAI(
                    api_key=deployment.creds.token,
                    base_url=deployment.endpoint,
                )
            raise ProviderNotFoundError(deployment)
        case _:
            raise ProviderNotFoundError(deployment)
